package com.migao.config.listener;


import com.migao.config.constant.SystemConstant;
import com.migao.config.properties.SystemProperties;
import com.migao.util.FileUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.ApplicationListener;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.event.ContextRefreshedEvent;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;


@Configuration
@Slf4j
public class UpdateFileStoreDirListener implements ApplicationListener<ContextRefreshedEvent> {

    @Resource
    private SystemProperties systemProperties;

    @Override
    public void onApplicationEvent(ContextRefreshedEvent contextRefreshedEvent) {
        log.debug("重启完成后重置文件下载、存储路径");
        String applicationRootPath = FileUtils.getApplicationRootPath();
        systemProperties.setApplicationRootPath(applicationRootPath);
        //重置下载url
        String downLoadPrefix = StringUtils.join(
                systemProperties.getDownloadPrefix(),
                SystemConstant.URI_CONCAT_SYMBOL,
                systemProperties.getFileStoreDir(),
                SystemConstant.URI_CONCAT_SYMBOL
        );
        systemProperties.setDownloadPrefix(
                downLoadPrefix
        );
        //重置文件下载路径
        String fileStoreDirPath = StringUtils.join(
                systemProperties.getApplicationRootPath(),
                File.separator,
                systemProperties.getFileStoreDir(),
                File.separator
        );
        systemProperties.setFileStoreDirPath(
                fileStoreDirPath
        );
        String tempDir = StringUtils.join(
                systemProperties.getApplicationRootPath(),
                File.separator,
                systemProperties.getFileStoreDir(),
                File.separator,
                systemProperties.getFileTempDir(),
                File.separator
        );
        //zip文件解压路径
        String zipOutDir = StringUtils.join(
                systemProperties.getApplicationRootPath(),
                File.separator,
                systemProperties.getFileStoreDir(),
                File.separator,
                systemProperties.getFileZipDir(),
                File.separator
        );
        systemProperties.setFileZipPath(
                zipOutDir
        );
        FileUtils.mkdirs(fileStoreDirPath);
        FileUtils.mkdirs(zipOutDir);
        FileUtils.mkdirs(tempDir);
    }
}
